/*
 *  Name: mgscores.c
 *  Author: Vincent Lefevre
 *  Version: 1.01
 *  Date: 20 / 02 / 1995
 *
 *  Merge scores
 *
 *  Usage: mgscores <source rulefile> <destination rulefile>
 */

#include <stdio.h>
#include <string.h>

enum error { ERR_USAGE = 1, ERR_RULEFILE, ERR_FOPEN, ERR_FCLOSE, ERR_FSEEK,
             ERR_FREAD, ERR_FWRITE, ERR_INSUFMEM, ERR_NAME };

typedef struct
{ char name[24];
  unsigned long score;
  unsigned long date;
}  Score;

void copy(Score *dest, Score *srce)
{
  short i = 7;

  do
    *((long *) dest)++ = *((long *) srce)++;
  while (--i >= 0);
}

Score s[3][4*255];

main(int argc, char *argv[])
{
  short i, q;
  FILE *f;
  long t;
  char name[2][24];
  unsigned char n[3][4];
  Score *s0, *s1, *s2;

  if (argc != 3)
    { fprintf(stderr,
              "Usage: mgscores <source rulefile> <destination rulefile>\n");
      return ERR_USAGE; }

  for(i=1;i<=2;i++)
    { if (!(f = fopen(argv[i], "rb")))
        { open_error:
          fprintf(stderr, "Cannot open file %s\n", argv[i]);
          return ERR_FOPEN; }
      if (!fread(&t, 4, 1, f) || t != 'RUL3')
        { rulefile_error:
          fprintf(stderr, "File %s is not a rule file\n", argv[i]);
          return ERR_RULEFILE; }
      if (fseek(f, 0x16, SEEK_SET))
        { seek_error:
          fprintf(stderr, "Seek error in file %s\n", argv[i]);
          return ERR_FSEEK; }
      if (!fread(&name[i-1], 24, 1, f))
        { read_error:
          fprintf(stderr, "Read error in file %s\n", argv[i]);
          return ERR_FREAD; }
      if (fseek(f, 0x36, SEEK_SET)) goto seek_error;
      if (!fread(n[i-1], 4 * i, 1, f)) goto read_error;
      if (fseek(f, 0x6A, SEEK_SET)) goto seek_error;
      if (!fread(&t, 4, 1, f)) goto read_error;
      if (t <= 0) goto rulefile_error;
      if (fseek(f, t, SEEK_SET)) goto seek_error;
      if ((q = n[i-1][0] + n[i-1][1] + n[i-1][2] + n[i-1][3])
       && !fread(s[i], sizeof(Score) * q, 1, f)) goto read_error;
      if (fclose(f))
        { close_error:
          fprintf(stderr, "Cannot close file %s\n", argv[i]);
          return ERR_FCLOSE; }
    }

  if (strcmp(&name[0], &name[1]))
    { fprintf(stderr, "The rule files do not have the same name\n");
      return ERR_NAME; }

  q = 0;
  s1 = &s[1][0];
  s2 = &s[2][0];
  for(i=0;i<4;i++)
    { short j = 0, k = 0, m;
      s0 = &s[0][q];
      for(m=0;m<n[2][i];m++)
        if (k < n[1][i])
          { if (j < n[0][i] && s1[j].score >= s2[k].score)
              { if (s1[j].score > s2[k].score
                 || s1[j].date < s2[k].date) goto cpy1;
                if (s1[j].date == s2[k].date
                 && !strcmp(s1[j].name, s2[k].name)) j++;
              }
            copy(&s0[m], &s2[k++]);
          }
        else if (j < n[0][i])
          cpy1: copy(&s0[m], &s1[j++]);
        else
          n[2][i] = m;
      s1 += n[0][i];
      s2 += n[1][i];
      q += n[2][i];
    }

  if (!(f = fopen(argv[2], "r+b"))) goto open_error;
  if (fseek(f, 0x36, SEEK_SET)) goto seek_error;
  if (!fwrite(n[2], 4, 1, f))
    { write_error:
      fprintf(stderr, "Write error in file %s\n", argv[2]);
      return ERR_FWRITE; }
  if (fseek(f, t, SEEK_SET)) goto seek_error;
  if (!fwrite(s[0], sizeof(Score) * q, 1, f)) goto write_error;
  if (fclose(f)) goto close_error;

  return 0;
}
